/* --------------------------------------------------------------------------
 * Copyright 1992-1993 by Forschungszentrum Informatik (FZI)
 *
 * You can use and distribute this software under the terms of the license
 * version 1 you should have received along with this software.
 * If not or if you want additional information, write to
 * Forschungszentrum Informatik, "STONE", Haid-und-Neu-Strasse 10-14,
 * D-76131 Karlsruhe, Germany.
 *
 * --------------------------------------------------------------------------
 * Interface definitions for the C --> OBST Interface: Utilities
 * --------------------------------------------------------------------------
 * ORIGINAL: D. Theobald				DATE: 14/2/93
 * --------------------------------------------------------------------------
 */
/* tclOBST LIBRARY MODULE */

#define OBST_IMP_STDCONST
#define OBST_IMP_MALLOC
#define OBST_IMP_STRINGOP
#define OBST_IMP_SORT_SEARCH
#include "obst_stdinc.h"

#include "_c2obst.h"


/*
 * Map strings into an unique storage representation.
 * Two strings resulting from this function are equal, iff their addresses
 * are identical.
 * It is an error to modify a string resulting from this function.
 * The result string will not be identical to the argument string, i.e. the
 * argument string might be modified afterwards.
 *
 * (Compact version used here - see test program for a more readable form.
 *  Remark: These routines do not deserve to be type-checked by a C++ compiler.)
 */
static int strcmpfct (s1ptr, s2ptr)
   _qsort_const_ void* s1ptr;
   _qsort_const_ void* s2ptr;
{
   return strcmp(*(char**)s1ptr, *(char**)s2ptr);
}
 
char* obst_uniqueString (str)
   const char* str;
{
   static char* *rootptr = NULL;
   static char* first_node;
   static char* *nptr    = &first_node;
          char* *found;
 
   *nptr = (char*)str;
   if (nptr == (found = *(char***)tsearch ((_qsort_const_ void*)nptr,
					   (void**)&rootptr,
                                           (qsort_cmpfct_t*)strcmpfct)))
   {  *nptr = obst_strdup(str);
      nptr  = (char* *)malloc ((size_t)sizeof(char*));
   }
   return *found;
}

/* -------------------------------------------------------------------- */

#define check_type(tp,tpname,wanted) \
   if (sizeof(tp) != (wanted)) \
      obst_err_raise (err_SYS, \
		      "inconsistent C mapping for C++ type", tpname, FALSE)

/*
 * Initialize the C interface with a C routine and check the consistency
 * of the mapping of the C++ types to C types.
 */
void _c2obst_Cinit()
{  static int initialized = 0;
   if (initialized) return;
   initialized = 1;

   check_type (sos_Char,      "sos_Char",      1);	/* This check is of */
   check_type (sos_Int,       "sos_Int",       4);	/* course rather    */
   check_type (sos_Cstring,   "sos_Cstring",   4);	/* rough; but we    */
   check_type (sos_Pointer,   "sos_Pointer",   4);	/* assume, that the */
   check_type (sos_Offset,    "sos_Offset",    4);	/* C type has the   */
   check_type (cci_Fun,       "cci_Fun",       4);	/* right flavor.    */
   check_type (sos_Container, "sos_Container", 4);
   check_type (sos_Object,    "sos_Object",    OBST_OBJ_SIZE);
}
